import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:tencent_calls_engine/tencent_calls_engine.dart';
import 'package:tencent_calls_uikit/src/platform/tuicall_kit_platform_interface.dart';
import 'package:tencent_calls_uikit/src/tuicall_data.dart';
import 'package:tencent_calls_uikit/src/tuicall_engine_manager.dart';
import 'package:tencent_calls_uikit/src/tuicall_navigator_observer.dart';
import 'package:tencent_calls_uikit/src/tuicall_state.dart';
import 'package:tencent_calls_uikit/tuicall_kit.dart';

class MethodChannelTUICallKit extends TUICallKitPlatform {
  MethodChannelTUICallKit() {
    methodChannel.setMethodCallHandler((call) async {
      debugPrint(
          "CallHandler method:${call.method}, arguments:${call.arguments}");
      if (call.method == "gotoCallingPage") {
        _gotoCallingPage();
      } else if (call.method == "handleCallReceived") {
        _handleCallReceived();
      } else if (call.method == "enableFloatWindow") {
        var enable = call.arguments['enable'];
        _handleEnableFloatWindow(enable);
      } else if (call.method == "groupCall") {
        var groupId = call.arguments['groupId'];
        var userIdList = List<String>.from(call.arguments['userIdList']);
        TUICallMediaType mediaType =
            TUICallMediaType.values[call.arguments['mediaType']];
        _handleGroupCall(groupId, userIdList, mediaType);
      } else if (call.method == "call") {
        var userId = call.arguments['userId'];
        TUICallMediaType mediaType =
            TUICallMediaType.values[call.arguments['mediaType']];
        _handleCall(userId, mediaType);
      } else if (call.method == "handleLoginSuccess") {
        var userId = call.arguments['userId'];
        var sdkAppId = call.arguments['sdkAppId'];
        var userSig = call.arguments['userSig'];
        _handleLoginSuccess(userId, sdkAppId, userSig);
      } else if (call.method == "handleLogoutSuccess") {
        _handleLogoutSuccess();
      } else {
        debugPrint("flutter: MethodNotImplemented ${call.method}");
      }
    });
  }

  @visibleForTesting
  final methodChannel = const MethodChannel('tuicall_kit');

  @override
  Future<void> startForegroundService() async {
    await methodChannel.invokeMethod('startForegroundService', {});
  }

  @override
  Future<void> stopForegroundService() async {
    await methodChannel.invokeMethod('stopForegroundService', {});
  }

  @override
  Future<void> startRing(String filePath) async {
    await methodChannel.invokeMethod('startRing', {"filePath": filePath});
  }

  @override
  Future<void> stopRing() async {
    await methodChannel.invokeMethod('stopRing', {});
  }

  @override
  Future<void> updateCallStateToNative() async {
    methodChannel.invokeMethod('updateCallStateToNative', {
      'selfUser': TUICallState.instance.selfUser.toJson(),
      'remoteUser': TUICallState.instance.remoteUserList.isNotEmpty
          ? TUICallState.instance.remoteUserList[0].toJson()
          : User().toJson(),
      'scene': TUICallState.instance.scene.index,
      'mediaType': TUICallState.instance.mediaType.index,
      'startTime': TUICallState.instance.startTime,
      'camera': TUICallState.instance.camera.index
    });
  }

  @override
  Future<void> startFloatWindow() async {
    await methodChannel.invokeMethod('startFloatWindow', {});
  }

  @override
  Future<void> stopFloatWindow() async {
    await methodChannel.invokeMethod('stopFloatWindow', {});
  }

  @override
  Future<bool> hasFloatPermission() async {
    return await methodChannel.invokeMethod('hasFloatPermission', {});
  }

  @override
  Future<bool> isAppInForeground() async {
    return await methodChannel.invokeMethod('isAppInForeground', {});
  }

  @override
  Future<bool> moveAppToFront(String event) async {
    return await methodChannel.invokeMethod('moveAppToFront', {"event": event});
  }

  void _gotoCallingPage() {
    TUICallKitNavigatorObserver.getInstance().enterCallingPage();
  }

  void _handleCallReceived() {
    TUICallState.instance.handleCallReceived(
        TUICallState.instance.caller.id,
        TUICallState.instance.calleeIdList,
        TUICallState.instance.groupId,
        TUICallState.instance.mediaType);
  }

  void _handleEnableFloatWindow(enable) {
    TUICallKit.instance.enableFloatWindow(enable);
  }

  void _handleGroupCall(
      groupId, List<String> userIdList, TUICallMediaType mediaType) {
    TUICallKit.instance.groupCall(groupId, userIdList, mediaType);
  }

  void _handleCall(userId, TUICallMediaType mediaType) {
    TUICallKit.instance.call(userId, mediaType);
  }

  void _handleLoginSuccess(userId, sdkAppId, userSig) {
    TUICallState.instance.registerEngineObserver();
    TUICallEngienManager.instance.initEngine(sdkAppId, userId, userSig);
    TUICallEngienManager.instance.adaptiveComponentReport();
  }

  void _handleLogoutSuccess() {
    TUICallEngine.instance.unInit();
    TUICallState.instance.unRegisterEngineObserver();
  }
}
