import 'dart:io';

import 'package:flutter/material.dart';
import 'package:tencent_calls_engine/tencent_calls_engine.dart';
import 'package:tencent_calls_uikit/src/i18n/i18n_utils.dart';
import 'package:tencent_calls_uikit/src/tuicall_engine_manager.dart';
import 'package:tencent_calls_uikit/src/tuicall_state.dart';
import 'package:tencent_calls_uikit/src/ui/widget/common/extent_button.dart';
import 'package:tencent_calls_uikit/src/utils/event_bus.dart';
import 'package:tencent_calls_uikit/src/utils/permission_request.dart';
import 'package:tencent_cloud_uikit_core/tencent_cloud_uikit_core.dart';

class GroupFunctionWidget {
  static Widget buildFunctionWidget(Function close) {
    if (TUICallStatus.waiting == TUICallState.instance.selfUser.callStatus) {
      if (TUICallRole.caller == TUICallState.instance.selfUser.callRole) {
        if (TUICallMediaType.audio == TUICallState.instance.mediaType) {
          return _buildAudioCallerWaitingAndAcceptedView(close);
        } else {
          return _buildVideoCallerAndCalleeAcceptedView(close);
        }
      } else {
        return _buildAudioAndVideoCalleeWaitingView(close);
      }
    } else if (TUICallStatus.accept ==
        TUICallState.instance.selfUser.callStatus) {
      if (TUICallMediaType.audio == TUICallState.instance.mediaType) {
        return _buildAudioCallerWaitingAndAcceptedView(close);
      } else {
        return _buildVideoCallerAndCalleeAcceptedView(close);
      }
    } else {
      return Container();
    }
  }

  static Widget _buildAudioCallerWaitingAndAcceptedView(Function close) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: [
        ExtendButton(
          imgUrl: TUICallState.instance.isMicrophoneMute
              ? "assets/images/mute_on.png"
              : "assets/images/mute.png",
          tips: CallKit_t("麦克风"),
          textColor: _getTextColor(),
          imgHieght: 52,
          onTap: () {
            _handleSwitchMic();
          },
        ),
        ExtendButton(
          imgUrl: "assets/images/hangup.png",
          tips: CallKit_t("挂断"),
          textColor: _getTextColor(),
          imgHieght: 64,
          onTap: () {
            _handleHangUp(close);
          },
        ),
        ExtendButton(
          imgUrl: TUICallState.instance.audioDevice ==
                  TUIAudioPlaybackDevice.speakerphone
              ? "assets/images/handsfree_on.png"
              : "assets/images/handsfree.png",
          tips: TUICallState.instance.audioDevice ==
                  TUIAudioPlaybackDevice.speakerphone
              ? CallKit_t("扬声器")
              : CallKit_t("听筒"),
          imgHieght: 52,
          onTap: () {
            _handleSwitchAudioDevice();
          },
        ),
      ],
    );
  }

  static Widget _buildVideoCallerAndCalleeAcceptedView(Function close) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: [
            ExtendButton(
              imgUrl: TUICallState.instance.isMicrophoneMute
                  ? "assets/images/mute_on.png"
                  : "assets/images/mute.png",
              tips: CallKit_t("麦克风"),
              textColor: _getTextColor(),
              imgHieght: 52,
              onTap: () {
                _handleSwitchMic();
              },
            ),
            ExtendButton(
              imgUrl: TUICallState.instance.audioDevice ==
                      TUIAudioPlaybackDevice.speakerphone
                  ? "assets/images/handsfree_on.png"
                  : "assets/images/handsfree.png",
              tips: TUICallState.instance.audioDevice ==
                      TUIAudioPlaybackDevice.speakerphone
                  ? CallKit_t("扬声器")
                  : CallKit_t("听筒"),
              textColor: _getTextColor(),
              imgHieght: 52,
              onTap: () {
                _handleSwitchAudioDevice();
              },
            ),
            ExtendButton(
              imgUrl: TUICallState.instance.isCameraOpen
                  ? "assets/images/camera_on.png"
                  : "assets/images/camera_off.png",
              tips: CallKit_t("摄像头"),
              textColor: _getTextColor(),
              imgHieght: 52,
              onTap: () {
                _handleOpenCloseCamera();
              },
            ),
          ],
        ),
        const SizedBox(
          height: 25,
        ),
        Row(mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: [
          const SizedBox(
            width: 42,
          ),
          ExtendButton(
            imgUrl: "assets/images/hangup.png",
            tips: CallKit_t("挂断"),
            textColor: _getTextColor(),
            imgHieght: 64,
            onTap: () {
              _handleHangUp(close);
            },
          ),
          ExtendButton(
            imgUrl: "assets/images/switch_camera.png",
            tips: CallKit_t(" "),
            textColor: _getTextColor(),
            imgHieght: 42,
            onTap: () {
              _handleSwitchCamera();
            },
          ),
        ]),
      ],
    );
  }

  static Widget _buildAudioAndVideoCalleeWaitingView(Function close) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: [
        ExtendButton(
          imgUrl: "assets/images/hangup.png",
          tips: CallKit_t("挂断"),
          textColor: _getTextColor(),
          imgHieght: 64,
          onTap: () {
            _handleReject(close);
          },
        ),
        ExtendButton(
          imgUrl: "assets/images/dialing.png",
          tips: CallKit_t("接听"),
          textColor: _getTextColor(),
          imgHieght: 64,
          onTap: () {
            _handleAccept();
          },
        ),
      ],
    );
  }

  static _handleSwitchMic() async {
    if (TUICallState.instance.isMicrophoneMute) {
      TUICallState.instance.isMicrophoneMute = false;
      await TUICallEngienManager.instance.openMicrophone();
    } else {
      TUICallState.instance.isMicrophoneMute = true;
      await TUICallEngienManager.instance.closeMicrophone();
    }
    eventBus.notify(setStateEvent);
  }

  static _handleSwitchAudioDevice() async {
    if (TUICallState.instance.audioDevice == TUIAudioPlaybackDevice.earpiece) {
      TUICallState.instance.audioDevice = TUIAudioPlaybackDevice.speakerphone;
    } else {
      TUICallState.instance.audioDevice = TUIAudioPlaybackDevice.earpiece;
    }
    await TUICallEngienManager.instance
        .selectAudioPlaybackDevice(TUICallState.instance.audioDevice);
    eventBus.notify(setStateEvent);
  }

  static _handleHangUp(Function close) async {
    await TUICallEngienManager.instance.hangup();
    close();
  }

  static _handleReject(Function close) async {
    await TUICallEngienManager.instance.reject();
    close();
  }

  static _handleAccept() async {
    var permissionRequestResult;
    if (Platform.isAndroid) {
      permissionRequestResult = await PermissionRequest.checkCallingPermission(
          TUICallState.instance.mediaType);
    }
    if (permissionRequestResult == TUIPermissionResult.granted ||
        Platform.isIOS) {
      await TUICallEngienManager.instance.accept();
      TUICallState.instance.selfUser.callStatus = TUICallStatus.accept;
    } else {
      TUIToast.show(content: CallKit_t("新通话呼入，但因权限不足，无法接听。请确认摄像头/麦克风权限已开启。"));
    }
    eventBus.notify(setStateEvent);
  }

  static void _handleOpenCloseCamera() async {
    TUICallState.instance.isCameraOpen = !TUICallState.instance.isCameraOpen;
    if (TUICallState.instance.isCameraOpen) {
      await TUICallEngienManager.instance.openCamera(
          TUICallState.instance.camera, TUICallState.instance.selfUser.viewID);
    } else {
      await TUICallEngienManager.instance.closeCamera();
    }
    eventBus.notify(setStateEvent);
  }

  static void _handleSwitchCamera() async {
    if (TUICamera.front == TUICallState.instance.camera) {
      TUICallState.instance.camera = TUICamera.back;
    } else {
      TUICallState.instance.camera = TUICamera.front;
    }
    await TUICallEngienManager.instance
        .switchCamera(TUICallState.instance.camera);
    eventBus.notify(setStateEvent);
  }

  static Color _getTextColor() {
    return (TUICallMediaType.audio == TUICallState.instance.mediaType)
        ? Colors.black
        : Colors.white;
  }
}
